import os
from os.path import exists
from icheckin import constants
from collections import OrderedDict

#################################################

def _help():
	print(_usage()+'\n')
	print(_details())

def _save(cred, test=False):
	if not test:
		path = constants.PATH
	else:
		path = constants.TEST_PATH
	with open(path, 'w') as file:
		file.write(constants.KEYWORD_1+'\n')
		file.write(cred[0]+'\n\n')
		file.write(constants.KEYWORD_2+'\n')
		file.write(cred[1]+'\n')
	print('success: credentials saved to %s' % path)

def _remove(test=False):
	if not test:
		path = constants.PATH
	else:
		path = constants.TEST_PATH
	if exists(path):
		os.remove(path)
		print('success: %s removed' % path)
	else:
		print('error: %s does not exist' % path)

commands = OrderedDict()
commands['-h'] = {
	'alt'	: '--help',
	'info': 'Display help for command-line tools',
	'action': _help,
	'params': []
}
commands['-s'] = {
	'alt': '--save',
	'info': 'Save credentials',
	'action': _save,
	'params': ['student id', 'password']	
}
commands['-r'] = {
	'alt': '--remove',
	'info': 'Remove saved credentials',
	'action': _remove,
	'params': []	
}

#################################################

def process(args):
	''' 
	Process commands and carry out operations
	-> True	: Command executed
	-> False : Error with command or parameters
	''' 
	# Extract command and parameters
	cmd = args[0]
	if len(args) > 1:
		params = args[1:]
	else:
		params = []
	# Check command
	exists = False
	for command in commands.keys():
		if cmd == command:
			exists = True
		elif cmd == commands[command]['alt']:
			cmd = command
			exists = True
	if exists:
		# Check parameters
		parameters = commands[cmd]['params']
		if len(params) == len(parameters):
			# Execute command
			action = commands[cmd]['action']
			if len(params) == 0:
				action()
			else:
				action(params)
			return True
		else:
			# Invalid parameters
			print('error: invalid parameters')
			error = 'command: icheckin %s' % cmd
			for parameter in parameters:
				error += ' <%s>' % parameter
			print(error)
			return False
	else:
		# Invalid command
		print('unknown command: %s' % cmd)
		print(_usage())
		return False

def _usage():
	''' -> str	: Formatted usage ''' 
	head = 'usage: icheckin'
	usage = head
	for command in commands.keys():
		chunk = '%s' % command
		alt = commands[command]['alt']
		if alt != '':
			chunk += ' | %s' % alt
		params = commands[command]['params']
		for i in range(len(params)):
			chunk += ' <%s>' % params[i]
		chunk = ' [%s]' % chunk
		# Formatting
		buffer = usage + chunk
		if len(buffer.split('\n')[-1]) > 80:
			usage = '%s\n%s%s' % (usage, ' '*len(head), chunk)
		else:
			usage = buffer
	return usage

def _details():
	''' -> str	: Formatted details'''
	names = []
	infos = []
	for command in commands.keys():
		name = '   %s' % command
		alt = commands[command]['alt']
		if alt != '':
			name += ', %s' % alt
		names.append(name)
		infos.append(commands[command]['info'])
	a = max(list(map(lambda x: len(x), names))) + 3
	lines = []
	for i in range(len(names)):
		# Right pad name
		b = len(names[i])
		pad = a - b
		line = '%s%s' % (names[i], ' '*(pad-1))
		# Format info
		right = 80 - a
		for word in infos[i].split():
			buffer = '%s %s' % (line, word)
			if len(buffer) > 80:
				lines.append(line+'\n')
				line = '%s%s' % (' '*a, word)
			else:
				line = buffer
		lines.append(line+'\n')
	return ''.join(lines).strip('\n')




import sys; sys.path.append('../icheckin')
from icheckin import commands, constants
from collections import OrderedDict
from imp import reload
from os.path import exists
import os

def setup_module(module):
	def _a():
		print('A')

	def _b(args):
		print('B')

	def _c(args):
		print('C')

	def _d():
		print('D')

	def _e(args):
		print('E')

	def _f(args):
		print('F')

	commands.commands = OrderedDict()
	commands.commands['-a'] = {
		'alt'	: '--apple',
		'info': 'Apple',
		'action': _a,
		'params': []
	}
	commands.commands['-b'] = {
		'alt': '--boy',
		'info': 'Boy',
		'action': _b,
		'params': ['one']	
	}
	commands.commands['-c'] = {
		'alt': '',
		'info': 'Cat',
		'action': _c,
		'params': ['one one', 'two']	
	}
	commands.commands['-d'] = {
		'alt': '',
		'info': 'Dog',
		'action': _d,
		'params': []	
	}
	commands.commands['-e'] = {
		'alt': '--elephant',
		'info': 'Elephant',
		'action': _e,
		'params': ['one one', 'two', 'three three three']	
	}
	commands.commands['-f'] = {
		'alt': '--fool',
		'info': 'Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool ' + 
			'Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool ' +
			'Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool ' + 
			'Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool ' + 
			'Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool ',
		'action': _f,
		'params': ['one one', 'two', 'three three three']	
	}

	global usageMsg
	usageMsg = '''
usage: icheckin [-a | --apple] [-b | --boy <one>] [-c <one one> <two>] [-d]
                [-e | --elephant <one one> <two> <three three three>]
                [-f | --fool <one one> <two> <three three three>]
'''.strip('\n')
	global detailsMsg
	detailsMsg = '''
   -a, --apple      Apple
   -b, --boy        Boy
   -c               Cat
   -d               Dog
   -e, --elephant   Elephant
   -f, --fool       Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool
                    Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool
                    Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool
                    Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool
                    Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool Fool
                    Fool Fool Fool Fool
'''.strip('\n')
	global parametersMsg
	parametersMsg = '''
error: invalid parameters
'''.strip('\n')
	global commandMsgA
	commandMsgA = '''
command: icheckin -a
'''.strip('\n')
	global commandMsgB
	commandMsgB = '''
command: icheckin -b <one>
'''.strip('\n')
	global commandMsgF
	commandMsgF = '''
command: icheckin -f <one one> <two> <three three three>
'''.strip('\n')
	global unknownMsgZ
	unknownMsgZ = '''
unknown command: -z
'''.strip('\n')
	global unknownMsgFool
	unknownMsgFool = '''
unknown command: -fool
'''.strip('\n')
	global credentials
	credentials = '''
%s
123

%s
456
''' % (constants.KEYWORD_1, constants.KEYWORD_2)
	credentials = credentials.strip('\n')

def test_process_valid_command_no_parameter(capfd):
	assert commands.process(['-a']) == True
	out, err = capfd.readouterr()
	assert out == 'A' + '\n'
	assert commands.process(['--apple']) == True
	out, err = capfd.readouterr()
	assert out == 'A' + '\n'
	assert commands.process(['-d']) == True
	out, err = capfd.readouterr()
	assert out == 'D' + '\n'

def test_process_valid_command_valid_parameters(capfd):
	assert commands.process(['-b', '1']) == True
	out, err = capfd.readouterr()
	assert out == 'B' + '\n'
	assert commands.process(['--boy', '1']) == True
	out, err = capfd.readouterr()
	assert out == 'B' + '\n'
	assert commands.process(['-c', '1', '2']) == True
	out, err = capfd.readouterr()
	assert out == 'C' + '\n'
	assert commands.process(['-e', '1', '2', '3']) == True
	out, err = capfd.readouterr()
	assert out == 'E' + '\n'
	assert commands.process(['--elephant', '1', '2', '3']) == True
	out, err = capfd.readouterr()
	assert out == 'E' + '\n'
	assert commands.process(['-f', '1', '2', '3']) == True
	out, err = capfd.readouterr()
	assert out == 'F' + '\n'
	assert commands.process(['--fool', '1', '2', '3']) == True
	out, err = capfd.readouterr()
	assert out == 'F' + '\n'

def test_process_valid_command_invalid_parameters(capfd):
	assert commands.process(['-a', '1']) == False
	out, err = capfd.readouterr()
	assert out == parametersMsg + '\n' + commandMsgA + '\n'
	assert commands.process(['-b']) == False
	out, err = capfd.readouterr()
	assert out == parametersMsg + '\n' + commandMsgB + '\n'
	assert commands.process(['-f', '1']) == False
	out, err = capfd.readouterr()
	assert out == parametersMsg + '\n' + commandMsgF + '\n'

def test_process_invalid_command_no_parameter(capfd):
	assert commands.process(['-z']) == False
	out, err = capfd.readouterr()
	assert out == unknownMsgZ + '\n' + usageMsg + '\n'
	assert commands.process(['-fool']) == False
	out, err = capfd.readouterr()
	assert out == unknownMsgFool + '\n' + usageMsg + '\n'

def test_process_invalid_command_with_parameters(capfd):
	assert commands.process(['-z', '1' , '2']) == False
	out, err = capfd.readouterr()
	assert out == unknownMsgZ + '\n' + usageMsg + '\n'
	assert commands.process(['-fool', '1' , '2']) == False
	out, err = capfd.readouterr()
	assert out == unknownMsgFool + '\n' + usageMsg + '\n'

def test_display_help(capfd):
	assert commands._usage() == usageMsg
	assert commands._details() == detailsMsg
	commands._help()
	out, err = capfd.readouterr()
	assert out == usageMsg + '\n\n' + detailsMsg + '\n'

def test_save_new_file(capfd):
	reload(commands)
	if exists(constants.TEST_PATH):
		os.remove(constants.TEST_PATH)
	commands._save(['123', '456'], test=True)
	out, err = capfd.readouterr()
	assert out == 'success: credentials saved to %s' % constants.TEST_PATH + '\n'
	with open(constants.TEST_PATH, 'r') as file:
		assert file.readlines() == list(map(lambda x: x+'\n', 
			credentials.split('\n')))

def test_save_overwrite_file(capfd):
	reload(commands)
	with open(constants.TEST_PATH, 'w') as file:
		file.write('000')
	commands._save(['123', '456'], test=True)
	out, err = capfd.readouterr()
	assert out == 'success: credentials saved to %s' % constants.TEST_PATH + '\n'
	with open(constants.TEST_PATH, 'r') as file:
		assert file.readlines() == list(map(lambda x: x+'\n', 
			credentials.split('\n')))

def test_remove_existing_file(capfd):
	reload(commands)
	if not exists(constants.TEST_PATH):
		with open(constants.TEST_PATH, 'w') as file:
			file.write('000')
	commands._remove(test=True)
	out, err = capfd.readouterr()
	assert out == 'success: %s removed' % constants.TEST_PATH + '\n'
	assert exists(constants.TEST_PATH) == False

def test_remove_nonexisting_file(capfd):
	reload(commands)
	if exists(constants.TEST_PATH):
		os.remove(constants.TEST_PATH)	
	commands._remove(test=True)
	out, err = capfd.readouterr()
	assert out == 'error: %s does not exist' % constants.TEST_PATH + '\n'
	assert exists(constants.TEST_PATH) == False

def test_help_action():
	reload(commands)
	assert commands.commands['-h']['action'] == commands._help

def test_save_action():
	reload(commands)
	assert commands.commands['-s']['action'] == commands._save

def test_remove_action():
	reload(commands)
	assert commands.commands['-r']['action'] == commands._remove

def teardown_module(module):
	if exists(constants.TEST_PATH):
		os.remove(constants.TEST_PATH)




import requests
import sys
from getpass import getpass
from os.path import exists
from icheckin import constants, commands

def main():
	# Command-line tools
	if len(sys.argv) > 1:
		commands.process(sys.argv[1:])
		sys.exit()

	# Check for updates
	update = _checkUpdate(constants.UPDATES, constants.VERSION)
	if update is not None:
		print(update+'\n')

	# Start a session
	session = requests.Session()

	extracted = True
	while True:
		if exists(constants.PATH):
			# Extract credentials
			credentials = _extractCredentials(constants.PATH, constants.KEYWORD_1, 
				constants.KEYWORD_2)
			if type(credentials) == str:
				print(credentials)
				sys.exit()
		else:
			# Prompt for credentials
			credentials = (input('Student ID: '), getpass('Password: '))
			extracted = False
			if '' in credentials:
				sys.exit()
		# Login to iZone
		payload = {
			'form_action': 'submitted',
			'student_uid': credentials[0],
			'password': credentials[1],
		}
		login = _loginIzone(session, constants.LOGIN, payload)
		if login is None:
			break
		else:
			print(login)
			if extracted:
				sys.exit()
			else:
				print()

	# Save credentials
	if not extracted:
		print('Save your credentials?')
		while True:
			save = input('(Y to save / Enter to ignore) ')
			if save == '':
				break
			if save.upper() == 'Y':
				commands.process(['-s', credentials[0], credentials[1]])
				break
		print()

	while True:	
		# Prompt for code
		code = input('Code: ')
		if code == '':
			sys.exit()
		# Check for SunwayEdu Wi-Fi
		sunway = _checkSunwayWifi(constants.WIFI)
		if sunway is not None:
			print(sunway+'\n')
			continue
		# Check in with code
		checkin = _checkinCode(session, constants.CHECKIN, code)
		if checkin is None:
			print('Successfully checked in.')
			break
		else:
			print(checkin+'\n')

def _checkUpdate(u, v):
	''' 
	-> None	: Version is latest or checking failed
	-> str 	: Update message for when a newer version is available 
	'''
	try:
		r = requests.get(u, timeout=2)
	except (requests.ConnectionError, requests.ConnectTimeout, 
		requests.ReadTimeout):
		return None
	else:
		latestVersion = r.json()['latest_version']
		if v != latestVersion:
			return 'A newer version (%s) is available.\n' % latestVersion +\
				'$ pip install --upgrade icheckin'
		else:
			return None

def _extractCredentials(p, k1, k2):
	''' 
	-> tuple	: Student ID and password 
	-> str 	: Error message
	'''
	with open(p, 'r') as file:
		lines = list(map(str.strip, file.readlines()))
		# Check format of file content
		if len(lines) == 5 and lines[0] == k1 and lines[3] == k2:
			return (lines[1], lines[4])
		else:
			return 'error: something is wrong with %s\n\
				Save your credentials again or remove your credentials.' % p

def _loginIzone(s, lg, pl):
	''' 
	-> None	: Login successful
	-> str 	: Error message 
	'''
	try:
		r = s.post(lg, data=pl)
	except requests.ConnectionError:
		return 'No internet connection.'
	if r.history:
		return None
	else:
		return 'Invalid credentials.'

def _checkSunwayWifi(w):
	''' 
	-> None	: Connected to SunwayEdu Wi-Fi
	-> str 	: Error message
	'''
	try:
		r = requests.get(w, timeout=2)
	except requests.ConnectTimeout:
		return 'Not connected to SunwayEdu Wi-Fi.'
	except requests.ConnectionError:
		return 'No internet connection.'
	else:
		return None

def _checkinCode(s, ci, co):
	try:
		r = s.post(ci, data={'checkin_code': co}, timeout=2)
	except (requests.ReadTimeout, requests.ConnectionError):
		return 'No internet connection.'
	if 'Checkin code not valid.' in r.text or\
		'The specified URL cannot be found.' in r.text:
		return 'Invalid code.'
	else:
		return None

if __name__ == '__main__':
	main()


from os.path import join, expanduser, realpath, dirname

# Version
VERSION = '1.2.1'

# URLs
UPDATES = 'http://chunkhang.pythonanywhere.com/icheckin/api'
LOGIN = 'https://izone.sunway.edu.my/login'
WIFI = 'https://icheckin.sunway.edu.my/otp/CheckIn/isAlive/' +\
	'CuNv9UV2rXg4WtAsXUPNptg6gWQTZ52w'
CHECKIN = 'https://izone.sunway.edu.my/icheckin/iCheckinNowWithCode'

# Credentials
KEYWORD_1 = '[Student ID]'
KEYWORD_2 = '[Password]'
PATH = join(expanduser("~"), '.icheckin-credentials')
TEST_PATH = join(dirname(dirname(realpath(__file__))), 'test/.test-credentials')